/****************************************************************************
* MeshLab                                                           o o     *
* An extendible mesh processor                                    o     o   *
*                                                                _   O  _   *
* Copyright(C) 2005, 2006                                          \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/****************************************************************************
  History

$Log: glarea.h,v $
Revision 1.11  2005/11/27 03:50:58  glvertex
- Added method setColorMode
- Now getCurrentRenderMode is inline method

Revision 1.10  2005/11/26 16:53:54  glvertex
getRenderState --> getCurrentRenderMode

Revision 1.9  2005/11/26 14:09:15  alemochi
Added double side lighting and fancy lighting (working only double side+fancy)

Revision 1.8  2005/11/25 11:55:59  alemochi
Added function to Enable/Disable lighting (work in progress)

Revision 1.7  2005/11/24 01:38:36  cignoni
Added new plugins intefaces, tested with shownormal render mode

Revision 1.6  2005/11/20 19:30:50  glvertex
- Added lighting parameters (still working on...)
- Added logging events

Revision 1.5  2005/11/19 12:14:20  glvertex
Some cleanup and renaming

Revision 1.4  2005/11/18 18:25:35  alemochi
Rename function in glArea.h

Revision 1.3  2005/11/18 18:10:28  alemochi
Aggiunto slot cambiare la modalita' di rendering

Revision 1.2  2005/11/17 14:54:27  glvertex
Some little changes to allow differents rendering modes (not working yet)

Revision 1.1  2005/10/18 10:38:02  cignoni
First rough version. It simply load a mesh.

****************************************************************************/
#ifndef GLAREA_H
#define GLAREA_H
#include <wrap/gl/trimesh.h>
#include <wrap/gui/trackball.h>

#include <QWidget>
#include <QGLWidget>

#include "GLLogStream.h"

#include "meshmodel.h"
#include "interfaces.h"


enum LightingModel{LDOUBLE,LFANCY};

class RenderMode
{
public:
  	vcg::GLW::DrawMode	drawMode;
  	vcg::GLW::ColorMode	drawColor;
  	vcg::GLW::TextureMode	drawTexture;

    bool Lighting;
    bool BackFaceCull;
    bool DoubleSideLighting;  
    bool FancyLighting;
    bool CastShadow;
    vcg::Point3f LightDir;
		RenderMode()
		{
			Lighting=true;
			BackFaceCull=false;
			DoubleSideLighting=false;
			FancyLighting=false;
			CastShadow=false;
    }
};




class MeshModel;
class GLArea : public QGLWidget
{
	Q_OBJECT

public:
	GLArea(QWidget *parent = 0);
	~GLArea(){}
	
	MeshModel *mm;
	vcg::Trackball trackball;
	GLLogStream log;

	QSize minimumSizeHint() const;
	QSize sizeHint() const;
	
	const RenderMode & getCurrentRenderMode()	{return rm;}
	void setDrawMode(vcg::GLW::DrawMode mode);
	void setColorMode(vcg::GLW::ColorMode mode);
	void setLight(bool state);
	void setLightMode(bool state,LightingModel lmode);
  inline void RenderLight();
  MeshRenderInterface *iRender; 
  QString iRenderString;
protected:
	
	void initializeGL();
	void paintGL();
	void resizeGL(int width, int height);
	void mousePressEvent(QMouseEvent *event);
	void mouseMoveEvent(QMouseEvent *event);
	void mouseReleaseEvent(QMouseEvent *event);
	void wheelEvent(QWheelEvent*e);

  
private:
  RenderMode rm;

};


#endif
